function status = animate(soln, option);

global robot settings wd
l = robot.params.l;

status = 0;

if nargin ~= 2 | option ~= 0 & option ~= 1 & option ~= 2
    option = 0; % Reset axes as default
end

% Set the default erase mode
if option == 2
    style = 'normal';
else
    style = 'xor';
end

% Parse input
t = [soln(:).x];
q = [soln(:).y];
if istood
    stance = q(1,:)';
    nstance = q(2,:)';
%     roll = zeros(size(stance))';
elseif isthreed
    stance = q(2,:)';
    nstance = q(3,:)';
    roll = q(1,:)';
end

% Translate the angles into locations in 3D Cartesian space
vect = [0 0 l]'; % The vector in the non/stance leg reference frames
Rs = zeros(3, 3);
Rns = zeros(3, 3);
Rr = zeros(3, 3);
stanceleg = zeros(3, length(t));
nstanceleg = zeros(3, length(t));
impactpt = zeros(3, length(t));
j = 1; k = 1;
for i=1:length(t)
    % The following rotation matrices are from leg ref frame to world frame
    Rs = [1 0              0              
          0 cos(stance(i)) -sin(stance(i))
          0 sin(stance(i)) cos(stance(i))];
    Rns = [1 0               0                 
           0 cos(nstance(i)) sin(nstance(i))
           0 sin(nstance(i)) -cos(nstance(i))];
    if isthreed       
        Rr = [cos(roll(i))  0 -sin(roll(i))
              0             1 0
              sin(roll(i))  0 cos(roll(i))];
    end

    % Keep the indices of the points where the robot hits the ground
    if i > 2
        if t(i) == t(i-1)
            impactpt(:,i) = nstanceleg(:,i-1);
            adjust(j) = i;
            j = j + 1;
        else
            impactpt(:,i) = impactpt(:,i-1);
        end
    end

    % Assemble points using rigid-body kinematics
    stanceleg(:,i) = impactpt(:,i) + Rr*(Rs*vect);
    nstanceleg(:,i) = stanceleg(:,i) + Rr*(Rns*vect);
end

% Assemble the robot
sx = [impactpt(1,:)' stanceleg(1,:)']; % Stance points
sy = [impactpt(2,:)' stanceleg(2,:)'];
sz = [impactpt(3,:)' stanceleg(3,:)'];
nsx = [stanceleg(1,:)' nstanceleg(1,:)']; % Nonstance points
nsy = [stanceleg(2,:)' nstanceleg(2,:)'];
nsz = [stanceleg(3,:)' nstanceleg(3,:)'];
groundx = [nstanceleg(1,:)' impactpt(1,:)']; % Ground points (slope)
groundy = [nstanceleg(2,:)' impactpt(2,:)'];
groundz = [nstanceleg(3,:)' impactpt(3,:)'];
projx = [sx nsx]; % Shadow on the ground (z = 0)
projy = [sy nsy];
twody = [sy nsy]; % 2D projection (x = .5 surface)
twodz = [sz nsz];

% Create figure
screenSize = get(0, 'ScreenSize');
if option == 0
    width = screenSize(3)/2;
    left = screenSize(3)/4;
    bottom = screenSize(4)/4;
    height = screenSize(4)/2;
elseif option == 1 | option == 2
    width = screenSize(3);
    left = 0;
    bottom = screenSize(4)/4;
    height = screenSize(4)/1.5;
end
figure('Position', [left bottom width height])
set(gcf, 'Color', [1 1 1])

if option ~= 2 % Then we use line instead of 
    

% Create graphics objects
sleg = line(sx(1,:), sy(1,:), sz(1,:), 'EraseMode', style, 'Marker', '.', 'Color', 'b');
nsleg = line(nsx(1,:), nsy(1,:), nsz(1,:), 'EraseMode', style, 'Marker', '.', 'Color', [0 .5 0]);
ground = line(groundx(1,:), groundy(1,:), groundz(1,:), 'EraseMode', 'none', 'Marker', '.', 'Color', 'k');
proj = line(projx(1,:), projy(1,:), [0 0 0 0], 'EraseMode', style, 'Color', [.5 .5 .5]);
twod = line([0 0 0 0], twody(1,:), twodz(1,:), 'EraseMode', style, 'Color', [.5 .25 .25]);

get(sleg, 'XData')
nsleg
ground
proj

pause

% if option == 2
%     set(proj, 'EraseMode', 'none')
%     set(twod, 'EraseMode', 'none')
% end

% Adjust axes
if option == 0
    stride = norm([sy(1,:) nsy(1,:) groundy(1,:)]) - min(sy(1,:)); 
    ymin = min(nsy(1,:)) - 1.5; ymax = norm(nsy(1,:)) + 1.5;
elseif option == 1 | option == 2
    stride = 0;
    ymin = min(nsy(1,:)) - 1.5; ymax = norm (nsy(1,:)) + 4;
end
xmin = min(groundx(1,:)) - .5; xmax = max(groundx(1,:)) + .5;
% zmin = min(sz(1,:)) - .1; zmax = norm(sz(1,:)) + .1;
zmin = 0; zmax = norm(sz(1,:)) + .1;
axis([ xmin xmax ymin ymax zmin zmax ])
axis fill

% Set the 3D viewpoint
if istood
    view([-90 0])
elseif isthreed
    view([-20 30])
    box on
end

% Draw and create movie
j = 1;
xlabel('x')
ylabel('y')
zlabel('z')
for i=2:length(t)
    % Make adjustments on impact
    if length(find(adjust == i)) ~= 0
        j = j + 1;
        
        % Swap colors
        if option == 2
            % Draw the ground
            if i == 1
                set(ground, 'XData', groundx(i,:), 'YData', groundy(i,:), 'ZData', groundz(i,:))
            else
                ground = plot3(groundx(i,:), groundy(i,:), groundz(i,:))
            end
        else
            % Draw the ground
%             set(ground, 'XData', groundx(i,:), 'YData', groundy(i,:), 'ZData', groundz(i,:))
            ground = plot3(groundx(i,:), groundy(i,:), groundz(i,:))
%             if get(sleg, 'Color') == [0 0 1]
%                 set(sleg, 'Color', [0 .5 0]);
%                 set(nsleg, 'Color', [0 0 1]);
%             else
%                 set(sleg, 'Color', [0 0 1]);
%                 set(nsleg, 'Color', [0 .5 0]);
%             end
        end
        
        if option == 0
            stride = abs(max(groundy(i,:)) - min(groundy(i,:))) - .05; % minus .05 because it isn't realistic if it just walks in place.
            xmin = min(groundx(i,:)) - .5; xmax = norm(groundx(i,:)) + .5;
            ymin = ymin + stride; ymax = ymax + stride;
            axis([ xmin xmax ymin ymax zmin zmax ])
            axis fill
        end
        
    end
    
    % Some temporary image-saving options
%     t(i)
%     if t(i) > 1
%         drawnow
%         F = getframe(gcf);
%         imwrite(F.cdata, 'robot-pos-reset.jpg', 'jpg');
%         imwrite(F.cdata, 'robot-pos-reset.tiff', 'tiff');
%         imwrite(F.cdata, 'robot-pos-reset.png', 'png');
%         pause
%     end
    
    % Rotate viewpoint as you animate - computationally intensive!
%     azi = linspace(30, -30, size(t,1));
%     el = linspace(-25, 25, size(t,1));
%     view([azi(i) el(i)])

    if option == 2 
        if  length(find(adjust == i)) ~= 0
            hold on
            sleg = plot3(sx(i,:), sy(i,:), sz(i,:), 'Color', 'b');
            nsleg = plot3(nsx(i,:), nsy(i,:), nsz(i,:), 'Color', [0 .5 0]);
            proj = plot3(projx(i,:), projy(i,:), zeros(1,4));
            twod = plot3(xmax*ones(1,4), twody(i,:), twodz(i,:));
            if mod(j,2) == 1
                if get(sleg, 'Color') == [0 0 1]
                    set(sleg, 'Color', [0 .5 0]);
                    set(nsleg, 'Color', [0 0 1]);
                else
                    set(sleg, 'Color', [0 0 1]);
                    set(nsleg, 'Color', [0 .5 0]);
                end
            end
%             set(sleg, 'XData', sx(i,:), 'YData', sy(i,:), 'ZData', sz(i,:))
%             set(nsleg, 'XData', nsx(i,:), 'YData', nsy(i,:), 'ZData', nsz(i,:))
%             set(proj, 'XData', projx(i,:), 'YData', projy(i,:), 'ZData', [0 0 0 0])
%             set(twod, 'XData', xmax*ones(1,4), 'YData', twody(i,:), 'ZData', twodz(i,:))
            drawnow
        end
    else
        set(sleg, 'XData', sx(i,:), 'YData', sy(i,:), 'ZData', sz(i,:))
        set(nsleg, 'XData', nsx(i,:), 'YData', nsy(i,:), 'ZData', nsz(i,:))
        set(proj, 'XData', projx(i,:), 'YData', projy(i,:), 'ZData', [0 0 0 0])
        set(twod, 'XData', xmax*ones(1,4), 'YData', twody(i,:), 'ZData', twodz(i,:))  
%             sleg = plot3(sx(i,:), sy(i,:), sz(i,:), 'Color', 'b', 'EraseMode', style, 'Marker', '.', 'Color', 'b');
%             nsleg = plot3(nsx(i,:), nsy(i,:), nsz(i,:), 'Color', [0 .5 0], 'EraseMode', style, 'Marker', '.', 'Color', 'b');
%             proj = plot3(projx(i,:), projy(i,:), zeros(1,4), 'EraseMode', style, 'Marker', '.', 'Color', 'b');
%             twod = plot3(xmax*ones(1,4), twody(i,:), twodz(i,:), 'EraseMode', style, 'Marker', '.', 'Color', 'b');
            view([-90 0])
%             if mod(j,2) == 1
%                 if get(sleg, 'Color') == [0 0 1]
%                     set(sleg, 'Color', [0 .5 0]);
%                     set(nsleg, 'Color', [0 0 1]);
%                 else
%                     set(sleg, 'Color', [0 0 1]);
%                     set(nsleg, 'Color', [0 .5 0]);
%                 end
%             end
        drawnow
    end
end

% status = robotMovie;
% movie(robotMovie)

if option == 2
%         % Adjust axes
%     if option == 0
%         stride = norm([sy(1,:) nsy(1,:) groundy(1,:)]) - min(sy(1,:)); 
%         ymin = min(nsy(1,:)) - 1.5; ymax = norm(nsy(1,:)) + 1.5;
%     elseif option == 1 | option == 2
%         stride = 0;
%         ymin = min(nsy(1,:)) - 1.5; ymax = norm (nsy(1,:)) + 4;
%     end
%     xmin = min(groundx(1,:)) - .5; xmax = max(groundx(1,:)) + .5;
%     % zmin = min(sz(1,:)) - .1; zmax = norm(sz(1,:)) + .1;
%     zmin = 0; zmax = norm(sz(1,:)) + .1;
%     axis([ xmin xmax ymin ymax zmin zmax ])
%     axis fill
%     
%     % Set the 3D viewpoint
%     if strcmp(robot.dir(size(wd,2)-1:size(wd,2)), '2D')
%         view([-90 0])
%     elseif strcmp(robot.dir(size(wd,2)-1:size(wd,2)), '3D')
%         view([-20 30])
%         box on
%     end
    xlabel('x')
    ylabel('y')
    zlabel('z')
end
