
/* The following calls to this function are allowed:
 *
 * nrhs == 1:
 *      eqns(q)
 * nrhs == 2:
 *      eqns(t, q)
 *      eqns(q, flag)
 * nrhs == 3:
 *      eqns(t, q, flag)
 *
 * q = state vector, t = time, flag = 's', 'g', 'i', 't'.
 * Notice that the flag is always the last element, and follows the q 
 * vector no matter which argument the q vector is.
 *
 */

#include "math.h"
#include "mex.h"

void mexFunction( int nlhs, mxArray *plhs[], 
                    int nrhs, const mxArray *prhs[] ) {
    
/* ****************************** */
/* Declare pointers and variables */
/* ****************************** */
    const int dim = %* dimensions *%;
    char *paramNames[<* Length[parameters] *>] = <* parameters *>;
    mxArray *param, *robot;
    /* Model parameters */
    double <* StringReplace[ToString[parameters], {"{" -> "", "}" -> "", "\""->""}] *>;    
    double *tmp;
    double *q;
    double *out;
    double guard, traj;
    
    int spot = 0, i = 0;
    char flag; 
    q = calloc(dim, sizeof(double));
    if (q == NULL) dieHorribly(6);
    
/* ************ */
/* Parse inputs */
/* ************ */
    if (nrhs == 0) dieHorribly(0);
    
    /* Find and parse global model parameters */
    robot = mxGetField(mexGetVariablePtr("global", "robot"), 0, "params");
    for (i=0 ; i<<* Length[parameters] *> ; i++) {
        param = mxGetField(robot, 0, paramNames[i]);
        if (param == NULL) dieHorribly(4);
        switch(i) {
            %*
            casestring = "";
            For[i = Length[parameters], i > 0, i--,
                casestring = Append[{"case " <> ToString[i - 1] <> ": " <> 
                StringReplace[ToString[parameters[[i]]], {"{" -> "",
                                                          "}" -> "",
                                                          "\"" -> ""}] <> 
                " = *mxGetPr(param); break;"}, casestring]];
            StringReplace[ToString[casestring], {"{" -> "", 
                                                 "}" -> "", 
                                                 "\"" -> "",
                                                 "," ->"\n           "}]
            *%
            default: dieHorribly(-1);
        }
    }
    
    /* Find q vector */
    i = 0;
    spot = -1;
    for (i=0 ; i<nrhs ; i++) {
        if ((mxGetM(prhs[i]) == dim && mxGetN(prhs[i]) == 1) || 
            (mxGetM(prhs[i]) == 1   && mxGetN(prhs[i]) == dim)) {
            spot = i;
            break;
        }
    }
    
    /* Did we find an input vector? */
    if (spot == -1 || spot >= nrhs) {
        dieHorribly(1);
    }
    
    /* Parse q vector */
    for (i=0 ; i<dim ; i++) {
        q[i] = mxGetPr(prhs[spot])[i];
        if (mxIsNaN(q[i]) == 1 || mxIsInf(q[i]) == 1) {
            /* mexPrintf("eqns error: Input is NaN or Inf. Setting global variable 'invalidResults' to '1'.\n"); */
            putInvalid();
        }
        /* mexPrintf("q[%i] = %f\n", i, q[i]); */
    }
    
    /* Find and parse flag */
    if (spot+1 == nrhs-1 && mxIsChar(prhs[spot+1])) {
        flag = *mxGetChars(prhs[spot+1]);
    } else {
        flag = 's';
    }
    if (flag != 's' && flag != 'i' && flag != 'g' && flag != 't') dieHorribly(3);

    /*********************/
    /* Compute dynamics: */
    /*********************/
    if (flag == 's') {
        /* Allocate memory for output */
        tmp = mxCalloc(<*
        numLines = StringToStream[ ToString[
           CAssign[ %* Array[out, Floor[dimensions/2], Floor[dimensions/2]] *% , %* qdd *% ]
              ] ];
        value = ToString[Length[ReadList[numLines, String]]];
        Close[numLines];
        StringReplace[value, "," ->"\n"] *>, sizeof(double));
        plhs[0] = mxCreateDoubleMatrix(dim, 1, mxREAL);
        if (plhs[0] == NULL) dieHorribly(-1);
        out = mxGetPr(plhs[0]);  
        
<* CAssign[ %* Array[out, Floor[dimensions/2], Floor[dimensions/2]] *% , %* qdd *% ] *>
        
        /* The last half of the input is the first half of the output */
        for (i=0 ; i<<*Floor[dimensions/2]*> ; i++) {
            if (mxIsNaN(out[i+<*Floor[dimensions/2]*>]) == 1 || mxIsInf(out[i+<*Floor[dimensions/2]*>]) == 1) {
                /* mexPrintf("eqns error: Dynamics output is NaN or Inf. Setting global variable 'invalidResults' to '1'.\n");*/
                putInvalid();
            }
            out[i] = q[i+<*Floor[dimensions/2]*>];
        }
        
        /* Free temporary memory */
        mxFree(tmp);        
        
    /*******************/
    /* Compute impacts */
    /*******************/    
    } else if (flag == 'i') {
        /* Allocate memory for output */
        tmp = mxCalloc(<*
        numLines = StringToStream[ ToString[
           CAssign[ %* Array[out, Floor[dimensions/2], Floor[dimensions/2]] *% , %* qdimpact *% ]
              ] ];
        value = ToString[Length[ReadList[numLines, String]]];
        Close[numLines];
        StringReplace[value, "," ->"\n"] *>, sizeof(double));
        plhs[0] = mxCreateDoubleMatrix(dim, 1, mxREAL);
        if (plhs[0] == NULL) dieHorribly(6);
        out = mxGetPr(plhs[0]);

<* CAssign[ %* Array[out, Floor[dimensions/2], Floor[dimensions/2]] *% , %* qdimpact *% ] *>

        /* Check for valid output */
        for (i=<*Floor[dimensions/2]*> ; i<dim ; i++) {
            if (mxIsNaN(out[i]) == 1 || mxIsInf(out[i]) == 1) {
                /* mexPrintf("eqns error: Impact equations' output is NaN or Inf. Setting global variable 'invalidResults' to '1'.\n"); */
                putInvalid();
            }
        }
        
    /*************************/
    /* Apply transition map: */
    /*************************/
<* CAssign[ %* Array[out, Floor[dimensions/2], 0] *% , %* transition *% ] *>

        /* Free temporary memory */
        mxFree(tmp);

    /*****************/
    /* Compute guard */
    /*****************/
    } else if (flag == 'g') { 
<* CAssign[guard, guard] *>
        plhs[0] = mxCreateScalarDouble(guard);
        if (plhs[0] == NULL) dieHorribly(6);

    /********************************/
    /* Compute holonomic trajectory */
    /********************************/
    } else if (flag == 't') {
<* CAssign[traj, traj] *>
        plhs[0] = mxCreateScalarDouble(traj);
        if (plhs[0] == NULL) dieHorribly(6);
    }
    
/* ************************ */
/* Free up allocated memory */
/* ************************ */ 
    free(q);
}

void dieHorribly( int option ) {

    switch (option) {
        case 0:
            mexErrMsgTxt("eqns error:\nThe following calls are allowed:\nnrhs == 1:\n   eqns(q)\nnrhs == 2:\n   eqns(t, q)\n   eqns?(q, flag)\nnrhs == 3:\n   eqns(t, q, flag)\n\nt = time (but only if called by ode45), q = state vector, flag = 's' or 'i' or 'g'.\n");
            break;
        case 1:
            mexErrMsgTxt("eqns error:\nMake sure that the input state vector has the right dimensions, is either a column or row vector, and is the first or second argument.");
            break;
        case 2:
            mexErrMsgTxt("eqns error:\nWrong # of arguments enter 1, 2 or 3 inputs.\n");
            break;
        case 3:
            mexErrMsgTxt("eqns error:\nThe flag must be 's' to compute dynamics, 'i' to compute impact, or 'g' to compute the guard function.\n");
            break;
        case 4:
            mexErrMsgTxt("eqns error:\nDefine your global variables first.\n");
            break;
        case 5:
            mexErrMsgTxt("eqns error: You have NaN or Inf output.\n");
            break;
        case 6:
            mexErrMsgTxt("eqns error: Cannot allocate memory for calloc.\n");
            break;
        default:
            mexErrMsgTxt("eqns error: The sky is falling, or Eric made a logical typo.\n");
            break;
    }
}

void putInvalid() {
    mxArray *invalidValue;
    int success;
    double value = 1.;
    
    invalidValue = mxCreateScalarDouble(value);
    success = mexPutVariable("global", "invalidResults", invalidValue);
    if (success == 1) {
        dieHorribly(-1);
    }
}
