
function sol = step(q, tlast);
% function sol = step(q, tlast);
% step          Take 1 step, mindful of where we are wrt the guard.

global settings robot

if nargin == 1
    tlast = 0;
end

already_hit_guard = 0;
% Did we start on the guard?
if abs( guard(tlast, q) ) <= 1E-15
    already_hit_guard = 1;
    q = robot.eqns(q, 'i');
    if settings.debug
        fprintf('step warning:\nStarted on the guard, so applied reset map.\nNow integrating from %f', q);
    end
end

% Integrate until you hit the guard
tmax = settings.integration_time; % seconds.
sol = settings.integrator(robot.eqns, [tlast tlast+tmax], q, settings.ode_options);
if settings.debug
    [sol.xe sol.ye']
end

% The integration terminated. Why?

% ... either we didn't arrive at the guard
if size(sol.xe, 1) == 0 
    if settings.debug
        fprintf('The step took longer than %i seconds.\n', tmax);
    end
    return

% ... or we did arrive at the guard.
elseif abs( guard(sol.xe(end), sol.ye(:,end)) ) <= 1E-4
    if already_hit_guard == 1 % Don't apply the reset map
        if settings.debug
            fprintf('step warning:\nIntegration stopped on the guard, but we already applied the reset map before integrating.\n')
        end
    elseif already_hit_guard == 0 % Then apply the reset map
        sol.x = [sol.x sol.xe(end)];
        sol.y = [sol.y robot.eqns(sol.ye(:,end)', 'i')];
        if settings.debug
            fprintf('step notice:\nHit the guard after integrating, applied reset map.\n')
        end
    end
else
    if settings.stop_if_unstable == 1
        sol.ye = [];
        sol.xe = [];
%         error('step warning: Could not find the guard within %i seconds.', tmax);
    end    
end
