
function soln = walk(q, steps, graph)
% function soln = walk(q, steps, graph)
% walk          Walk forward "steps". "graph" is a string of characters.
% The default characters are 'd' and 'a[1|2]', so that you could have
%   graph = 'da1'
% for example.

if nargin == 2
    graph = '';
end

global settings robot

soln = repmat(struct('solver', '', 'extdata', [], 'x', [], 'y', [], ...
    'xe', [],'ye', [], 'ie', [], 'stats', [], 'idata', []), [steps 1]);

% Take several steps
ti = 0;
qi = q;
for i=1:steps
    soln(i) = step(qi, ti);
    if settings.stop_if_unstable == 1 
        if size( soln(i).ye, 1 ) == 0 % If you didn't make a step
            soln(i:end)=[];
            break
        end
    end
    ti = soln(i).x(end);
    qi = soln(i).y(:,end);
end

% Animate?
if size(regexp(graph, 'a'), 1) ~= 0
    if regexp(graph, 'a') == length(graph)
        option = 0;
    else
        option = graph(regexp(graph, 'a')+1);
    end
    animate(soln, option);
end

% Display dynamics?
if size(regexp(graph, 'd'))
    makefigs(soln, 'd');
end

% Display control input?
if size(regexp(graph, 'u'))
    makefigs(soln, 'u');
end

% Display error?
if size(regexp(graph, 'e'))
    makefigs(soln, 'e');
end

