(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     23158,        525]*)
(*NotebookOutlinePosition[     23822,        548]*)
(*  CellTagsIndexPosition[     23778,        544]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Execute first:", "Section"],

Cell[BoxData[{
    \(\(\(Needs["\<Format`\>"];\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(Off[General::spell, General::spell1];\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(SetOptions[CAssign, AssignTemporary \[Rule] {"\<tmp\>", Array}, 
        AssignMaxSize \[Rule] 1000, AssignTrig \[Rule] True, 
        AssignToArray \[Rule] {q, out}, AssignBreak \[Rule] False, 
        AssignOptimize \[Rule] False, 
        AssignReplace \[Rule] {"\<Beta\>" \[Rule] "\<beta\>"}, 
        AssignIndent \[Rule] "\<        \>"];\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(SetDirectory[
        ToFileName[\(("\<FileName\>" /. 
                NotebookInformation[
                  EvaluationNotebook[]])\)[\([1]\)]]];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["2D Models", "Section"],

Cell["\<\
All 2D models must have the guard, holonomic trajectory, impact equations \
(2), and expressions for the acceleration (2) defined, in addition to \
book-keeping lists qdd, qdimpact, and parameters. Also set the # of \
dimensions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
  "One\n",
  StyleBox["Pointfooted 2D model w/ masses at feet and knees.",
    FontSize->9]
}], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{\( (*\ Book - keeping\ variables\ and\ the\ like\ *) \), 
      "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"{", 
                
                RowBox[{\(\[Theta]s[t] := q[0]\), 
                  ",", \(\[Theta]ns[t] := q[1]\), ",", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["\[Theta]s", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}], 
                    ":=", \(q[2]\)}], ",", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["\[Theta]ns", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}], 
                    ":=", \(q[3]\)}]}], "}"}], ";"}], 
            "\[IndentingNewLine]", \(dimensions = 4;\), 
            "\[IndentingNewLine]", \(qdd = {"\<sdd\>", "\<nsdd\>"};\), 
            "\[IndentingNewLine]", \(qdimpact = {"\<sdimpact\>", "\<nsdimpact\
\>"};\), "\[IndentingNewLine]", \(Array[parameters, 6, 0];\), 
            "\[IndentingNewLine]", \(parameters = {"\<\"M\"\>", "\<\"Mp\"\>", \
\ "\<\"Mf\"\>", "\<\"g\"\>", "\<\"l\"\>", "\<\"slope\"\>"};\), 
            "\[IndentingNewLine]", 
            "\[IndentingNewLine]", \( (*\ Dynamics, \ impact, \ 
              transition\ equations\ *) \), 
            "\[IndentingNewLine]", \(transition\  = \ {\[Theta]ns[
                    t], \ \[Theta]s[t]};\), 
            "\[IndentingNewLine]", \(guard = 
              2\ slope + \[Theta]s[t] + \[Theta]ns[t]\)}], ";", 
          RowBox[{"traj", "=", 
            RowBox[{
              RowBox[{"l", " ", \(Sin[\[Theta]ns[t]]\), " ", 
                RowBox[{
                  SuperscriptBox["\[Theta]ns", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}]}], "-", 
              RowBox[{"l", " ", \(Sin[\[Theta]s[t]]\), " ", 
                RowBox[{
                  SuperscriptBox["\[Theta]s", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}]}]}]}], ";"}], 
        "\n", 
        RowBox[{
          RowBox[{"sdd", "=", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"2", " ", 
                  RowBox[{"(", 
                    
                    RowBox[{\(g\ \((M + 2\ Mf)\)\^2\ Sin[
                          2\ \[Theta]ns[t] - \[Theta]s[t]]\), 
                      "-", \(2\ g\ \((M\^2 + M\ \((5\ Mf + Mp)\) + 
                            2\ Mf\ \((Mf + 2\ Mp)\))\)\ Sin[\[Theta]s[t]]\), 
                      "+", 
                      RowBox[{"l", " ", \((M + 2\ Mf)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            
                            RowBox[{\((M + 4\ Mf)\), 
                              " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ",
                               
                              SuperscriptBox[
                                RowBox[{
                                  SuperscriptBox["\[Theta]ns", "\[Prime]",
                                    MultilineFunction->None], "[", "t", "]"}],
                                 "2"]}], "-", 
                            
                            RowBox[{\((M + 2\ Mf)\), 
                              " ", \(Sin[
                                2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\), " ",
                               
                              SuperscriptBox[
                                RowBox[{
                                  SuperscriptBox["\[Theta]s", "\[Prime]",
                                    MultilineFunction->None], "[", "t", "]"}],
                                 "2"]}]}], ")"}]}]}], ")"}]}], ")"}], 
              "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                      8\ Mf\ \((Mf + 2\ Mp)\) + 
                      2\ \((M + 2\ Mf)\)\^2\ Cos[
                          2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}]}], 
          ";"}], "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"nsdd", "=", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"2", " ", \((M + 2\ Mf)\), " ", 
                  RowBox[{"(", 
                    
                    RowBox[{\(2\ g\ \((M + Mf + Mp)\)\ Sin[\[Theta]ns[t]]\), 
                      "+", \(g\ \((3\ M + 
                            2\ \((Mf + Mp)\))\)\ Sin[\[Theta]ns[t] - 
                            2\ \[Theta]s[t]]\), "+", 
                      RowBox[{
                      "l", " ", \((M + 2\ Mf)\), 
                        " ", \(Sin[2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\), 
                        " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]ns", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}], "-", 
                      RowBox[{
                      "l", " ", \((5\ M + 4\ \((Mf + Mp)\))\), 
                        " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]s", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}]}], ")"}]}], ")"}], 
              "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                      8\ Mf\ \((Mf + 2\ Mp)\) + 
                      2\ \((M + 2\ Mf)\)\^2\ Cos[
                          2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}]}], 
          ";"}], "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"sdimpact", "=", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"M", " ", \((M + 4\ Mf)\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]ns", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}], "-", 
                  RowBox[{
                  "2", " ", \((M\^2 + 8\ Mf\ Mp + 2\ M\ \((3\ Mf + Mp)\))\), 
                    " ", \(Cos[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]s", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], 
              "/", \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                  8\ Mf\ \((Mf + 2\ Mp)\) + 
                  2\ \((M + 2\ Mf)\)\^2\ Cos[
                      2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\)}]}], ";", 
          RowBox[{"nsdimpact", "=", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                  "2", " ", "M", " ", \((M + 2\ Mf)\), 
                    " ", \(Cos[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]ns", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}], "+", 
                  
                  RowBox[{\((M\^2 - 4\ M\ Mf - 8\ Mf\ Mp - 
                        4\ \((M + 2\ Mf)\)\ \((M + Mp)\)\ Cos[
                            2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]s", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], 
              "/", \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                  8\ Mf\ \((Mf + 2\ Mp)\) + 
                  2\ \((M + 2\ Mf)\)\^2\ Cos[
                      2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\)}]}], ";"}], 
        "\[IndentingNewLine]", 
        "\[IndentingNewLine]", \(Splice[
            ToFileName[Directory[], "\<eqns.mc\>"], 
            ToFileName[Directory[], "\<2D\\1\\eqns.mctmp\>"], 
            Delimiters \[Rule] {"\<%*\>", "\<*%\>"}, 
            PageWidth \[Rule] 100000000, \ 
            FormatType \[Rule] OutputForm\ ];\), 
        "\[IndentingNewLine]", \(Splice[
            ToFileName[Directory[], "\<2D\\1\\eqns.mctmp\>"], 
            ToFileName[Directory[], "\<2D\\1\\eqns.c\>"], 
            Delimiters \[Rule] {"\<<*\>", "\<*>\>"}, 
            PageWidth \[Rule] 100000000, \ 
            FormatType \[Rule] OutputForm];\)}]}]], "Input"],

Cell[BoxData[
    \(Null\^8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(g\ \((M + Mf)\)\ \((M + 2\ Mf)\)\ Sin[
                    2\ \[Theta]ns[t] - \[Theta]s[t]]\), 
                "-", \(g\ \((M\^2 + 7\ M\ Mf + 6\ Mf\^2 + 2\ M\ Mp + 
                      8\ Mf\ Mp)\)\ Sin[\[Theta]s[t]]\), "+", 
                RowBox[{"2", " ", "l", " ", \((M + 2\ Mf)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      
                      RowBox[{\((M + 4\ Mf)\), 
                        " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]ns", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}], "-", 
                      
                      RowBox[{\((M + 2\ Mf)\), 
                        " ", \(Sin[2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\), 
                        " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]s", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}]}], ")"}]}]}], ")"}], 
            "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                    8\ Mf\ \((Mf + 2\ Mp)\) + 
                    2\ \((M + 2\ Mf)\)\^2\ Cos[
                        2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}], ",", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(g\ M\ \((3\ \((M + Mf)\) + 2\ Mp)\)\ Sin[\[Theta]ns[
                      t]]\), "+", \(2\ g\ \((M + 2\ Mf)\)\ \((M + Mf + 
                      Mp)\)\ Sin[\[Theta]ns[t] - 2\ \[Theta]s[t]]\), "+", 
                RowBox[{"2", " ", "l", " ", \((M + 2\ Mf)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      
                      RowBox[{\((M + 2\ Mf)\), 
                        " ", \(Sin[2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\), 
                        " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]ns", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}], "-", 
                      
                      RowBox[{\((5\ M + 4\ \((Mf + Mp)\))\), 
                        " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                        SuperscriptBox[
                          RowBox[{
                            SuperscriptBox["\[Theta]s", "\[Prime]",
                              MultilineFunction->None], "[", "t", "]"}], 
                          "2"]}]}], ")"}]}]}], ")"}], 
            "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                    8\ Mf\ \((Mf + 2\ Mp)\) + 
                    2\ \((M + 2\ Mf)\)\^2\ Cos[
                        2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}]}], 
        "}"}], "/.", \({M \[Rule] 2.5, \ Mp \[Rule] 5, \ Mf \[Rule] 2.5, 
        g \[Rule] 9.8, l \[Rule] 1, slope \[Rule] 0, q[0] \[Rule] 1, 
        q[1] \[Rule] 1, q[2] \[Rule] 1, q[3] \[Rule] 1}\)}]], "Input"],

Cell[BoxData[
    \({4.712237514924221`, 2.35611875746211`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Two\n",
  StyleBox["Currently the same ast One; just a place holder.",
    FontSize->9]
}], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{\( (*\ Book - keeping\ variables\ and\ the\ like\ *) \), 
      "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            
            RowBox[{\(\[Theta]s[t] := q[0]\), ",", \(\[Theta]ns[t] := q[1]\), 
              ",", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Theta]s", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                ":=", \(q[2]\)}], ",", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Theta]ns", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}], 
                ":=", \(q[3]\)}]}], "}"}], ";"}], 
        "\[IndentingNewLine]", \(dimensions = 4;\), 
        "\[IndentingNewLine]", \(qdd = {"\<sdd\>", "\<nsdd\>"};\), 
        "\[IndentingNewLine]", \(qdimpact = {"\<sdimpact\>", \
"\<nsdimpact\>"};\), "\[IndentingNewLine]", \(Array[parameters, 6, 0];\), 
        "\[IndentingNewLine]", \(parameters = {"\<\"M\"\>", "\<\"Mp\"\>", \ "\
\<\"Mf\"\>", "\<\"g\"\>", "\<\"l\"\>", "\<\"slope\"\>"};\), 
        "\[IndentingNewLine]", 
        "\[IndentingNewLine]", \( (*\ Dynamics, \ impact, \ 
          transition\ equations\ *) \), 
        "\[IndentingNewLine]", \(transition\  = \ {\[Theta]ns[t], \ \[Theta]s[
                t]};\), "\[IndentingNewLine]", 
        RowBox[{\(guard = 2\ slope + \[Theta]s[t] + \[Theta]ns[t]\), ";", 
          RowBox[{"traj", "=", 
            RowBox[{
              RowBox[{"l", " ", \(Sin[\[Theta]ns[t]]\), " ", 
                RowBox[{
                  SuperscriptBox["\[Theta]s", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}]}], "-", 
              RowBox[{"l", " ", \(Sin[\[Theta]s[t]]\), " ", 
                RowBox[{
                  SuperscriptBox["\[Theta]s", "\[Prime]",
                    MultilineFunction->None], "[", "t", "]"}]}]}]}], ";"}], 
        "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"sdd", "=", 
            RowBox[{
              RowBox[{"(", 
                
                RowBox[{\(g\ \((M + Mf)\)\ \((M + 2\ Mf)\)\ Sin[
                      2\ \[Theta]ns[t] - \[Theta]s[t]]\), 
                  "-", \(g\ \((M\^2 + 7\ M\ Mf + 6\ Mf\^2 + 2\ M\ Mp + 
                        8\ Mf\ Mp)\)\ Sin[\[Theta]s[t]]\), "+", 
                  RowBox[{"2", " ", "l", " ", \((M + 2\ Mf)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                        
                        RowBox[{\((M + 4\ Mf)\), 
                          " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                          SuperscriptBox[
                            RowBox[{
                              SuperscriptBox["\[Theta]ns", "\[Prime]",
                                MultilineFunction->None], "[", "t", "]"}], 
                            "2"]}], "-", 
                        
                        RowBox[{\((M + 2\ Mf)\), 
                          " ", \(Sin[2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\),
                           " ", 
                          SuperscriptBox[
                            RowBox[{
                              SuperscriptBox["\[Theta]s", "\[Prime]",
                                MultilineFunction->None], "[", "t", "]"}], 
                            "2"]}]}], ")"}]}]}], ")"}], 
              "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                      8\ Mf\ \((Mf + 2\ Mp)\) + 
                      2\ \((M + 2\ Mf)\)\^2\ Cos[
                          2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}]}], 
          ";"}], "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"nsdd", "=", 
            RowBox[{
              RowBox[{"(", 
                
                RowBox[{\(g\ M\ \((3\ \((M + Mf)\) + 2\ Mp)\)\ Sin[\[Theta]ns[
                        t]]\), 
                  "+", \(2\ g\ \((M + 2\ Mf)\)\ \((M + Mf + 
                        Mp)\)\ Sin[\[Theta]ns[t] - 2\ \[Theta]s[t]]\), "+", 
                  RowBox[{"2", " ", "l", " ", \((M + 2\ Mf)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                        
                        RowBox[{\((M + 2\ Mf)\), 
                          " ", \(Sin[2\ \((\[Theta]ns[t] - \[Theta]s[t])\)]\),
                           " ", 
                          SuperscriptBox[
                            RowBox[{
                              SuperscriptBox["\[Theta]ns", "\[Prime]",
                                MultilineFunction->None], "[", "t", "]"}], 
                            "2"]}], "-", 
                        
                        RowBox[{\((5\ M + 4\ \((Mf + Mp)\))\), 
                          " ", \(Sin[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                          SuperscriptBox[
                            RowBox[{
                              SuperscriptBox["\[Theta]s", "\[Prime]",
                                MultilineFunction->None], "[", "t", "]"}], 
                            "2"]}]}], ")"}]}]}], ")"}], 
              "/", \((l\ \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                      8\ Mf\ \((Mf + 2\ Mp)\) + 
                      2\ \((M + 2\ Mf)\)\^2\ Cos[
                          2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\))\)}]}], 
          ";"}], "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"sdimpact", "=", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"M", " ", \((M + 4\ Mf)\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]ns", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}], "-", 
                  RowBox[{
                  "2", " ", \((M\^2 + 8\ Mf\ Mp + 2\ M\ \((3\ Mf + Mp)\))\), 
                    " ", \(Cos[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]s", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], 
              "/", \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                  8\ Mf\ \((Mf + 2\ Mp)\) + 
                  2\ \((M + 2\ Mf)\)\^2\ Cos[
                      2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\)}]}], ";", 
          RowBox[{"nsdimpact", "=", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                  "2", " ", "M", " ", \((M + 2\ Mf)\), 
                    " ", \(Cos[\[Theta]ns[t] - \[Theta]s[t]]\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]ns", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}], "+", 
                  
                  RowBox[{\((M\^2 - 4\ M\ Mf - 8\ Mf\ Mp - 
                        4\ \((M + 2\ Mf)\)\ \((M + Mp)\)\ Cos[
                            2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\), " ", 
                    RowBox[{
                      SuperscriptBox["\[Theta]s", "\[Prime]",
                        MultilineFunction->None], "[", "t", "]"}]}]}], ")"}], 
              "/", \((\(-3\)\ M\^2 - 4\ M\ \((4\ Mf + Mp)\) - 
                  8\ Mf\ \((Mf + 2\ Mp)\) + 
                  2\ \((M + 2\ Mf)\)\^2\ Cos[
                      2\ \((\[Theta]ns[t] - \[Theta]s[t])\)])\)}]}], ";"}], 
        "\[IndentingNewLine]", 
        "\[IndentingNewLine]", \(Splice[
            ToFileName[Directory[], "\<eqns.mc\>"], 
            ToFileName[Directory[], "\<2D\\one\\eqns.mctmp\>"], 
            Delimiters \[Rule] {"\<%*\>", "\<*%\>"}, 
            PageWidth \[Rule] 100000000, \ 
            FormatType \[Rule] OutputForm\ ];\), 
        "\[IndentingNewLine]", \(Splice[
            ToFileName[Directory[], "\<2D\\one\\eqns.mctmp\>"], 
            ToFileName[Directory[], "\<2D\\one\\eqns.c\>"], 
            Delimiters \[Rule] {"\<<*\>", "\<*>\>"}, 
            PageWidth \[Rule] 100000000, \ 
            FormatType \[Rule] OutputForm];\)}]}]], "Input"],

Cell[BoxData[
    \(Null\^13\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 709}},
WindowSize->{632, 670},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 33, 0, 73, "Section"],
Cell[1812, 55, 796, 15, 266, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2645, 75, 28, 0, 73, "Section"],
Cell[2676, 77, 253, 5, 87, "Text"],

Cell[CellGroupData[{
Cell[2954, 86, 125, 4, 67, "Subsection"],

Cell[CellGroupData[{
Cell[3104, 94, 8297, 169, 945, "Input"],
Cell[11404, 265, 41, 1, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11482, 271, 3330, 67, 317, "Input"],
Cell[14815, 340, 73, 1, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[14937, 347, 124, 4, 75, "Subsection"],

Cell[CellGroupData[{
Cell[15086, 355, 7987, 162, 1128, "Input"],
Cell[23076, 519, 42, 1, 45, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

